@@ -1,14 +1,13 @@
 # -*- coding: utf-8 -*-
 """
 
-Usage
-======
+Code Usage
+----------
 
 The ``Parser`` class can be used to generate vakya parses thus:
 
 .. code-block:: python
 
-        from itertools import islice
         from sanskrit_parser import Parser
         string = "astyuttarasyAMdiSi"
         input_encoding = "SLP1"
@@ -41,7 +40,6 @@ This produces the output::
     uttarasyAm => (uttara#2, ['saptamIviBaktiH', 'strIliNgam', 'ekavacanam']) : viSezaRa of diSi
     diSi => (diS, ['saptamIviBaktiH', 'ekavacanam', 'strIliNgam']) : aDikaraRam of asti
 
-@author: avinashvarna
 """
 
 import time
@@ -138,6 +136,7 @@ class Parser():
               input_string: str,
               limit: int = 10,
               pre_segmented: bool = False,
+              dot_file=None,
               ):
         s = self._maybe_pre_segment(input_string, pre_segmented)
         logger.debug("Start Split")
@@ -148,6 +147,8 @@ class Parser():
             warnings.warn("No splits found. Please check the input to ensure there are no typos.")
             return None
         else:
+            if dot_file is not None:
+                graph.write_dot(dot_file)
             splits = graph.find_all_paths(max_paths=limit,
                                           sort=True,
                                           score=self.score)
