@@ -6,6 +6,7 @@ ac sandhi sutras generated through YAML processing
 """
 from sanskrit_parser.generator.process_yaml import process_yaml
 import yaml
+import os.path
 
 import logging
 logger = logging.getLogger(__name__)
@@ -13,10 +14,11 @@ logger = logging.getLogger(__name__)
 sutra_list = []
 sutra_dict = {}
 
-f = open("sutras.yaml", "r")
+f = open(os.path.join(os.path.dirname(__file__), "sutras.yaml"), "r")
 y = yaml.load(f, Loader=yaml.FullLoader)
-sutra_dict = process_yaml(y)    
+sutra_dict = process_yaml(y)
 sutra_list = sutra_dict.values()
-logger.info(sutra_dict)
+# logger.info(sutra_dict)
+f.close()
 
 __all__ = ["sutra_list", "sutra_dict"]
